@@ -4,7 +4,7 @@
 // the Cargo.lock file, but that doesn't work for publishing because the
 // lockfile is outside the crates/cli directory. So the workaround is to
 // verify this on the CI instead.
-import $ from "https://deno.land/x/dax@0.28.0/mod.ts";
+import $ from "https://deno.land/x/dax@0.33.0/mod.ts";
 
 $.logStep("Verifying wasmer-compiler version...");
 
@@ -12,13 +12,13 @@ const rootDir = $.path(import.meta).join("../../../../");
 const lockFile = rootDir.join("Cargo.lock");
 const rsFile = rootDir.join("crates/dprint/src/plugins/implementations/wasm/setup_wasm_plugin.rs");
 
-const lockText = lockFile.textSync();
+const lockText = lockFile.readTextSync();
 const version = lockText.match(/name = "wasmer-compiler"\r?\nversion = "([^"]+)"/m)![1];
 
 $.log(`Found ${version}`);
 
 const rsRegex = /pub const WASMER_COMPILER_VERSION: &str = "([^"]+)"/;
-const versionInRs = rsFile.textSync().match(rsRegex)![1];
+const versionInRs = rsFile.readTextSync().match(rsRegex)![1];
 $.log(`Existing version is: ${versionInRs}`);
 
 if (version === versionInRs) {
