@@ -1,4 +1,7 @@
 #![allow(clippy::bool_to_int_with_if)]
+#![deny(clippy::print_stderr)]
+#![deny(clippy::print_stdout)]
+#![deny(clippy::unused_async)]
 
 #[cfg(feature = "communication")]
 pub mod communication;
