@@ -1,6 +1,6 @@
 /// Checks for diagnostics and panics if it finds any.
 pub fn ensure_no_diagnostics<T: std::fmt::Debug>(diagnostics: &[T]) {
-  for diagnostic in diagnostics {
+  if let Some(diagnostic) = diagnostics.first() {
     panic!("Diagnostic error: {:?}", diagnostic);
   }
 }
