@@ -1,16 +1,58 @@
+use std::str::FromStr;
+
 use anyhow::bail;
 use anyhow::Result;
 use clap::ArgMatches;
 use thiserror::Error;
 
+use crate::environment::Environment;
 use crate::utils::LogLevel;
 use crate::utils::StdInReader;
 
+#[derive(Debug, Clone, Copy)]
+pub enum ConfigDiscovery {
+  Default,
+  IgnoreDescendants,
+  Disabled,
+}
+
+impl std::str::FromStr for ConfigDiscovery {
+  type Err = String;
+
+  fn from_str(s: &str) -> Result<Self, Self::Err> {
+    match s.to_ascii_lowercase().as_str() {
+      "default" | "true" | "1" => Ok(ConfigDiscovery::Default),
+      "false" | "0" => Ok(ConfigDiscovery::Disabled),
+      "ignore-descendants" => Ok(ConfigDiscovery::IgnoreDescendants),
+      _ => Err(format!("expected 'default', 'ignore-descendants' or 'false', got '{s}'")),
+    }
+  }
+}
+
+impl ConfigDiscovery {
+  pub fn traverse_ancestors(&self) -> bool {
+    match self {
+      ConfigDiscovery::Default => true,
+      ConfigDiscovery::IgnoreDescendants => true,
+      ConfigDiscovery::Disabled => false,
+    }
+  }
+
+  pub fn traverse_descendants(&self) -> bool {
+    match self {
+      ConfigDiscovery::Default => true,
+      ConfigDiscovery::IgnoreDescendants => false,
+      ConfigDiscovery::Disabled => false,
+    }
+  }
+}
+
 pub struct CliArgs {
   pub sub_command: SubCommand,
   pub log_level: LogLevel,
   pub plugins: Vec<String>,
   pub config: Option<String>,
+  config_discovery: Option<ConfigDiscovery>,
 }
 
 impl CliArgs {
@@ -21,6 +63,7 @@ impl CliArgs {
       log_level: LogLevel::Info,
       plugins: vec![],
       config: None,
+      config_discovery: None,
     }
   }
 
@@ -38,6 +81,19 @@ impl CliArgs {
       log_level: LogLevel::Info,
       config: None,
       plugins: Vec::new(),
+      config_discovery: None,
+    }
+  }
+
+  pub fn config_discovery(&self, env: &impl Environment) -> ConfigDiscovery {
+    match self.config_discovery {
+      Some(value) => value,
+      None => env
+        .env_var("DPRINT_CONFIG_DISCOVERY")
+        .as_ref()
+        .and_then(|value| value.to_str())
+        .and_then(|value| ConfigDiscovery::from_str(value).ok())
+        .unwrap_or(ConfigDiscovery::Default),
     }
   }
 }
@@ -103,6 +159,7 @@ pub struct CheckSubCommand {
   pub incremental: Option<bool>,
   pub list_different: bool,
   pub allow_no_files: bool,
+  pub only_staged: bool,
 }
 
 #[derive(Debug, PartialEq, Eq)]
@@ -112,6 +169,7 @@ pub struct FmtSubCommand {
   pub incremental: Option<bool>,
   pub enable_stable_format: bool,
   pub allow_no_files: bool,
+  pub only_staged: bool,
 }
 
 #[derive(Debug, PartialEq, Eq)]
@@ -160,6 +218,7 @@ pub struct FilePatternArgs {
   pub exclude_patterns: Vec<String>,
   pub exclude_pattern_overrides: Option<Vec<String>>,
   pub allow_node_modules: bool,
+  pub only_staged: bool,
 }
 
 #[derive(Debug, Error)]
@@ -207,13 +266,19 @@ fn inner_parse_args<TStdInReader: StdInReader>(args: Vec<String>, std_in_reader:
           patterns: parse_file_patterns(matches)?,
           incremental: parse_incremental(matches),
           enable_stable_format: !matches.get_flag("skip-stable-format"),
-          allow_no_files: matches.get_flag("allow-no-files"),
+          allow_no_files: if matches.get_flag("staged") {
+            true
+          } else {
+            matches.get_flag("allow-no-files")
+          },
+          only_staged: matches.get_flag("staged"),
         })
       }
     }
     ("check", matches) => SubCommand::Check(CheckSubCommand {
       patterns: parse_file_patterns(matches)?,
       incremental: parse_incremental(matches),
+      only_staged: matches.get_flag("staged"),
       list_different: matches.get_flag("list-different"),
       allow_no_files: matches.get_flag("allow-no-files"),
     }),
@@ -272,6 +337,7 @@ fn inner_parse_args<TStdInReader: StdInReader>(args: Vec<String>, std_in_reader:
       LogLevel::Info
     },
     config: matches.get_one::<String>("config").map(String::from),
+    config_discovery: matches.get_one::<ConfigDiscovery>("config-discovery").copied(),
     plugins: maybe_values_to_vec(matches.get_many("plugins")),
   })
 }
@@ -285,6 +351,7 @@ fn parse_file_patterns(matches: &ArgMatches) -> Result<FilePatternArgs> {
   }
 
   Ok(FilePatternArgs {
+    only_staged: matches.get_flag("staged"),
     allow_node_modules: matches.get_flag("allow-node-modules"),
     include_patterns: file_patterns,
     include_pattern_overrides: matches.get_many("includes-override").map(values_to_vec),
@@ -361,7 +428,7 @@ pub fn create_cli_parser(kind: CliArgParserKind) -> clap::Command {
   app = app
     .bin_name("dprint")
     .version(env!("CARGO_PKG_VERSION"))
-    .author("Copyright 2020-2023 by David Sherret")
+    .author("Copyright 2019 by David Sherret")
     .about("Auto-formats source code based on the specified plugins.")
     .override_usage("dprint <SUBCOMMAND> [OPTIONS] [--] [file patterns]...")
     .help_template(r#"{bin} {version}
@@ -381,12 +448,21 @@ OPTIONS:
 {options}
 
 ENVIRONMENT VARIABLES:
-  DPRINT_CACHE_DIR    Directory to store the dprint cache. Note that this
-                      directory may be periodically deleted by the CLI.
-  DPRINT_MAX_THREADS  Limit the number of threads dprint uses for
-                      formatting (ex. DPRINT_MAX_THREADS=4).
-  HTTPS_PROXY         Proxy to use when downloading plugins or configuration
-                      files (set HTTP_PROXY for HTTP).{after-help}"#)
+  DPRINT_CACHE_DIR     Directory to store the dprint cache. Note that this
+                       directory may be periodically deleted by the CLI.
+  DPRINT_MAX_THREADS   Limit the number of threads dprint uses for
+                       formatting (ex. DPRINT_MAX_THREADS=4).
+  DPRINT_CONFIG_DISCOVERY
+                       Sets the config discovery mode. Set to "false"/"0" to disable.
+  DPRINT_CERT          Load certificate authority from PEM encoded file.
+  DPRINT_TLS_CA_STORE  Comma-separated list of order dependent certificate stores.
+                       Possible values: "mozilla" and "system".
+                       Defaults to "mozilla,system".
+  DPRINT_IGNORE_CERTS  Unsafe way to get dprint to ignore certificates. Specify 1
+                       to ignore all certificates or a comma separated list of specific
+                       hosts to ignore (ex. dprint.dev,localhost,[::],127.0.0.1)
+  HTTPS_PROXY          Proxy to use when downloading plugins or configuration
+                       files (also supports HTTP_PROXY and NO_PROXY).{after-help}"#)
     .after_help(
             r#"GETTING STARTED:
   1. Navigate to the root directory of a code repository.
@@ -435,6 +511,7 @@ EXAMPLES:
             .num_args(0)
             .required(false)
         )
+        .add_only_staged_arg()
         .add_allow_no_files_arg()
         .arg(
           Arg::new("skip-stable-format")
@@ -452,6 +529,7 @@ EXAMPLES:
         .add_resolve_file_path_args()
         .add_incremental_arg()
         .add_allow_no_files_arg()
+        .add_only_staged_arg()
         .arg(
           Arg::new("list-different")
             .long("list-different")
@@ -486,6 +564,7 @@ EXAMPLES:
       Command::new("output-file-paths")
         .about("Prints the resolved file paths for the plugins based on the args and configuration.")
         .add_resolve_file_path_args()
+        .add_only_staged_arg()
     )
     .subcommand(
       Command::new("output-resolved-config")
@@ -496,6 +575,7 @@ EXAMPLES:
         .about("Prints the amount of time it takes to format each file. Use this for debugging.")
         .add_resolve_file_path_args()
         .add_allow_no_files_arg()
+        .add_only_staged_arg()
     )
     .subcommand(
       Command::new("clear-cache")
@@ -543,6 +623,17 @@ EXAMPLES:
         .global(true)
         .num_args(1)
     )
+    .arg(
+      Arg::new("config-discovery")
+        .long("config-discovery")
+        .help("Sets the config discovery mode. Set to `false` to completely disable.")
+        .global(true)
+        .value_parser(clap::value_parser!(ConfigDiscovery))
+        .value_name("BOOLEAN")
+        .num_args(1)
+        .require_equals(true)
+        .default_missing_value("true")
+    )
     .arg(
       Arg::new("plugins")
         .long("plugins")
@@ -587,6 +678,7 @@ trait ClapExtensions {
   fn add_resolve_file_path_args(self) -> Self;
   fn add_incremental_arg(self) -> Self;
   fn add_allow_no_files_arg(self) -> Self;
+  fn add_only_staged_arg(self) -> Self;
 }
 
 impl ClapExtensions for clap::Command {
@@ -649,6 +741,17 @@ impl ClapExtensions for clap::Command {
         .required(false),
     )
   }
+
+  fn add_only_staged_arg(self) -> Self {
+    use clap::Arg;
+    self.arg(
+      Arg::new("staged")
+        .long("staged")
+        .help("Format only the staged files.")
+        .num_args(0)
+        .required(false),
+    )
+  }
 }
 
 #[cfg(test)]
@@ -705,6 +808,21 @@ mod test {
     assert_eq!(fmt_cmd.incremental, Some(true));
   }
 
+  #[test]
+  fn staged_arg() {
+    let fmt_cmd = parse_fmt_sub_command(vec!["fmt"]).unwrap();
+    assert_eq!(fmt_cmd.only_staged, false);
+    let fmt_cmd = parse_fmt_sub_command(vec!["fmt", "--staged"]).unwrap();
+    assert_eq!(fmt_cmd.only_staged, true);
+  }
+
+  #[test]
+  fn no_files_arg() {
+    let fmt_cmd = parse_fmt_sub_command(vec!["fmt", "--staged"]).unwrap();
+    assert_eq!(fmt_cmd.only_staged, true);
+    assert_eq!(fmt_cmd.allow_no_files, true);
+  }
+
   fn parse_fmt_sub_command(args: Vec<&str>) -> Result<FmtSubCommand, ParseArgsError> {
     let args = test_args(args)?;
     match args.sub_command {
