@@ -282,16 +282,25 @@ impl<TEnvironment: Environment> Backend<TEnvironment> {
   }
 
   async fn send_format_request(&self, uri: &Url, request: EditorFormatRequest) -> LspResult<Option<Vec<TextEdit>>> {
+    let start_time = std::time::Instant::now();
+    log_debug!(self.environment, "Received format request for {}", uri);
     let mut drop_token = DropToken::new(request.token.clone());
     let result = self.send_format_request_inner(request).await;
     drop_token.completed();
-    match result {
+    let result = match result {
       Ok(value) => Ok(value),
       Err(err) => {
         log_error!(self.environment, "Failed formatting '{}': {:#}", uri, err);
         Ok(None)
       }
-    }
+    };
+    log_debug!(
+      self.environment,
+      "Finished format request for {} in {}ms",
+      uri,
+      start_time.elapsed().as_millis()
+    );
+    result
   }
 
   async fn send_format_request_inner(&self, request: EditorFormatRequest) -> Result<Option<Vec<TextEdit>>> {
@@ -482,6 +491,7 @@ mod test {
       })
       .initialize()
       .build();
+    environment.write_file(".gitignore", "gitignored_file.txt\ngitignored_dir").unwrap();
 
     environment.clone().run_in_runtime(async move {
       let (backend, recv_task, test_client) = setup_backend(environment.clone());
@@ -504,6 +514,16 @@ mod test {
             };
           }
 
+          macro_rules! did_close {
+            ($uri: ident) => {
+              backend
+                .did_close(DidCloseTextDocumentParams {
+                  text_document: TextDocumentIdentifier { uri: $uri.clone() },
+                })
+                .await;
+            };
+          }
+
           macro_rules! assert_format {
             ($uri: ident, $expected: expr) => {
               let result = backend
@@ -625,6 +645,16 @@ mod test {
           did_open!(file_uri, "testing");
           assert_format!(file_uri, None);
 
+          // ignores gitignored files
+          let file_uri = Url::parse("file:///gitignored_file.txt").unwrap();
+          did_open!(file_uri, "testing");
+          assert_format!(file_uri, None);
+
+          // ignores file in gitignored dir
+          let file_uri = Url::parse("file:///gitignored_dir/file.txt").unwrap();
+          did_open!(file_uri, "testing");
+          assert_format!(file_uri, None);
+
           // ignores excluded directory files
           let file_uri = Url::parse("file:///ignored-dir/file.txt").unwrap();
           did_open!(file_uri, "testing");
@@ -800,6 +830,67 @@ mod test {
             }])
           );
 
+          // now ensure formatting works with a sub folder config file that has different config
+          {
+            let mut config_file = TestConfigFileBuilder::new(environment.clone());
+            config_file.add_remote_wasm_plugin().add_remote_process_plugin();
+            environment.write_file("/other_config/dprint.json", &config_file.to_string()).unwrap();
+            let mut config_file = TestConfigFileBuilder::new(environment.clone());
+            config_file
+              .add_remote_wasm_plugin()
+              .add_remote_process_plugin()
+              .add_config_section("test-plugin", r#"{"ending": "asdf"}"#)
+              .add_config_section("testProcessPlugin", r#"{"ending": "asdf_ps"}"#);
+            environment.write_file("/other_config/sub/dprint.json", &config_file.to_string()).unwrap();
+          }
+
+          for _ in 0..2 {
+            let file_uri = Url::parse("file:///other_config/file.txt").unwrap();
+            did_open!(file_uri, "text");
+            assert_format!(
+              file_uri,
+              Some(vec![TextEdit {
+                range: Range::new(Position::new(0, 4), Position::new(0, 4)),
+                new_text: "_formatted".to_string()
+              }])
+            );
+            did_close!(file_uri);
+            // switching to a different config should still work fine
+            let file_uri = Url::parse("file:///other_config/sub/file.txt").unwrap();
+            did_open!(file_uri, "text");
+            assert_format!(
+              file_uri,
+              Some(vec![TextEdit {
+                range: Range::new(Position::new(0, 4), Position::new(0, 4)),
+                new_text: "_asdf".to_string()
+              }])
+            );
+            did_close!(file_uri);
+
+            // now try with a process plugin
+            let file_uri = Url::parse("file:///other_config/file.txt_ps").unwrap();
+            did_open!(file_uri, "text");
+            assert_format!(
+              file_uri,
+              Some(vec![TextEdit {
+                range: Range::new(Position::new(0, 4), Position::new(0, 4)),
+                new_text: "_formatted_process".to_string()
+              }])
+            );
+            did_close!(file_uri);
+            // switching to a different config should work fine as well
+            let file_uri = Url::parse("file:///other_config/sub/file.txt_ps").unwrap();
+            did_open!(file_uri, "text");
+            assert_format!(
+              file_uri,
+              Some(vec![TextEdit {
+                range: Range::new(Position::new(0, 4), Position::new(0, 4)),
+                new_text: "_asdf_ps".to_string()
+              }])
+            );
+            did_close!(file_uri);
+          }
+
           backend.shutdown().await.unwrap();
         }
       });
