@@ -8,7 +8,7 @@ pub async fn get_init_config_file_text(environment: &impl Environment) -> Result
   let info = match read_info_file(environment).await {
     Ok(info) => {
       // ok to only check wasm here because the configuration file is only ever initialized with wasm plugins
-      if info.plugin_system_schema_version != wasm::PLUGIN_SYSTEM_SCHEMA_VERSION {
+      if wasm::PLUGIN_SYSTEM_SCHEMA_VERSION < info.plugin_system_schema_version {
         log_error!(
           environment,
           concat!(
@@ -105,7 +105,7 @@ pub async fn get_init_config_file_text(environment: &impl Environment) -> Result
     }
     json_text.push_str("  ]\n}\n");
   } else {
-    json_text.push_str("  \"excludes\": [\n    \"**/node_modules\",\n    \"**/*-lock.json\"\n  ],\n");
+    json_text.push_str("  \"excludes\": [\n    \"**/*-lock.json\"\n  ],\n");
     json_text.push_str("  \"plugins\": [\n");
     json_text.push_str("    // specify plugin urls here\n");
     json_text.push_str("  ]\n}\n");
@@ -303,7 +303,6 @@ mod test {
         text,
         r#"{
   "excludes": [
-    "**/node_modules",
     "**/*-lock.json"
   ],
   "plugins": [
@@ -364,7 +363,7 @@ mod test {
   fn should_get_initialization_text_when_old_plugin_system() {
     let environment = TestEnvironmentBuilder::new()
       .with_info_file(|info| {
-        info.set_plugin_schema_version(9).add_plugin(TestInfoFilePlugin {
+        info.set_plugin_schema_version(999).add_plugin(TestInfoFilePlugin {
           name: "dprint-plugin-typescript".to_string(),
           version: "0.17.2".to_string(),
           url: "https://plugins.dprint.dev/typescript-0.17.2.wasm".to_string(),
@@ -382,7 +381,6 @@ mod test {
         text,
         r#"{
   "excludes": [
-    "**/node_modules",
     "**/*-lock.json"
   ],
   "plugins": [
@@ -395,7 +393,7 @@ mod test {
       expected_messages.push(concat!(
         "You are using an old version of dprint so the created config file may not be as helpful of a starting point. ",
         "Consider upgrading to support new plugins. ",
-        "Plugin system schema version is 3, latest is 9."
+        "Plugin system schema version is 4, latest is 999."
       ));
       assert_eq!(environment.take_stderr_messages(), expected_messages);
     });
