@@ -4,11 +4,13 @@ use anyhow::Result;
 use once_cell::sync::Lazy;
 use once_cell::sync::OnceCell;
 use parking_lot::Mutex;
+use std::ffi::OsString;
 use std::fs;
 use std::hash::Hash;
 use std::num::NonZeroUsize;
 use std::path::Path;
 use std::path::PathBuf;
+use std::process::Command;
 use std::sync::Arc;
 use std::time::SystemTime;
 use sysinfo::System;
@@ -29,8 +31,10 @@ use crate::utils::FastInsecureHasher;
 use crate::utils::LogLevel;
 use crate::utils::Logger;
 use crate::utils::LoggerOptions;
+use crate::utils::NoProxy;
 use crate::utils::ProgressBars;
 use crate::utils::RealUrlDownloader;
+use crate::utils::UnsafelyIgnoreCertificates;
 
 // cache the cwd because it's much faster than looking it up each time
 static CACHED_CWD: OnceCell<CanonicalizedPathBuf> = OnceCell::new();
@@ -56,7 +60,13 @@ impl RealEnvironment {
       log_level: options.log_level,
     }));
     let progress_bars = ProgressBars::new(&logger).map(Arc::new);
-    let url_downloader = Arc::new(RealUrlDownloader::new(progress_bars.clone(), logger.clone())?);
+    let no_proxy = NoProxy::from_env();
+    let url_downloader = Arc::new(RealUrlDownloader::new(
+      progress_bars.clone(),
+      logger.clone(),
+      no_proxy,
+      UnsafelyIgnoreCertificates::from_env(),
+    )?);
     let environment = RealEnvironment {
       url_downloader,
       logger,
@@ -105,6 +115,10 @@ impl Environment for RealEnvironment {
     true
   }
 
+  fn env_var(&self, name: &str) -> Option<OsString> {
+    std::env::var_os(name)
+  }
+
   fn read_file(&self, file_path: impl AsRef<Path>) -> Result<String> {
     Ok(String::from_utf8(self.read_file_bytes(file_path)?)?)
   }
@@ -118,6 +132,18 @@ impl Environment for RealEnvironment {
     }
   }
 
+  fn get_staged_files(&self) -> Result<Vec<PathBuf>> {
+    let output = Command::new("git")
+      .arg("diff")
+      .arg("--name-only")
+      .arg("--relative")
+      .arg("--staged")
+      .arg("--diff-filter=ACMR")
+      .output()?;
+
+    Ok(String::from_utf8_lossy(&output.stdout).lines().map(PathBuf::from).collect())
+  }
+
   fn write_file_bytes(&self, file_path: impl AsRef<Path>, bytes: &[u8]) -> Result<()> {
     log_debug!(self, "Writing file: {}", file_path.as_ref().display());
     #[allow(clippy::disallowed_methods)]
@@ -258,6 +284,10 @@ impl Environment for RealEnvironment {
     (*CACHE_DIR.as_ref().unwrap()).clone()
   }
 
+  fn get_home_dir(&self) -> Option<CanonicalizedPathBuf> {
+    dirs::home_dir().map(|path| self.canonicalize(path).unwrap())
+  }
+
   fn cpu_arch(&self) -> String {
     std::env::consts::ARCH.to_string()
   }
@@ -272,6 +302,7 @@ impl Environment for RealEnvironment {
   }
 
   fn max_threads(&self) -> usize {
+    #[allow(clippy::disallowed_methods)]
     resolve_max_threads(std::env::var("DPRINT_MAX_THREADS").ok(), std::thread::available_parallelism().ok())
   }
 
@@ -325,7 +356,7 @@ impl Environment for RealEnvironment {
     // need to also hash on the CPU features
     // https://github.com/dprint/dprint/issues/735
     let mut hash = FastInsecureHasher::default();
-    let mut features = wasmer::CpuFeature::for_host().into_iter().map(|c| c.to_string()).collect::<Vec<_>>();
+    let mut features = wasmer::sys::CpuFeature::for_host().into_iter().map(|c| c.to_string()).collect::<Vec<_>>();
     features.sort(); // ensure this is stable
     for feature in features {
       feature.hash(&mut hash);
@@ -343,7 +374,7 @@ impl Environment for RealEnvironment {
     let Ok(system) = dprint_core::async_runtime::spawn_blocking(move || {
       {
         let mut system = system.lock();
-        system.refresh_cpu();
+        system.refresh_cpu_usage();
       }
       system
     })
@@ -357,7 +388,7 @@ impl Environment for RealEnvironment {
 
     dprint_core::async_runtime::spawn_blocking(move || {
       let mut system = system.lock();
-      system.refresh_cpu();
+      system.refresh_cpu_usage();
       let utilization = system.cpus().iter().map(|c| c.cpu_usage()).sum::<f32>() / system.cpus().len() as f32;
       if utilization > 101f32 {
         0 // something wrong, so just return 0 for "cannot figure out cpu usage"
@@ -455,6 +486,7 @@ fn canonicalize_path(path: impl AsRef<Path>) -> Result<CanonicalizedPathBuf> {
 const CACHE_DIR_ENV_VAR_NAME: &str = "DPRINT_CACHE_DIR";
 
 static CACHE_DIR: Lazy<Result<CanonicalizedPathBuf>> = Lazy::new(|| {
+  #[allow(clippy::disallowed_methods)]
   let cache_dir = get_cache_dir_internal(|var_name| std::env::var(var_name).ok())?;
   #[allow(clippy::disallowed_methods)]
   std::fs::create_dir_all(&cache_dir)?;
