@@ -1,11 +1,13 @@
 use anyhow::anyhow;
 use anyhow::bail;
 use anyhow::Result;
+use once_cell::sync::Lazy;
 use parking_lot::Condvar;
 use parking_lot::Mutex;
 use path_clean::PathClean;
 use std::collections::HashMap;
 use std::collections::HashSet;
+use std::ffi::OsString;
 use std::future::Future;
 use std::io::Read;
 use std::io::Write;
@@ -21,6 +23,7 @@ use super::Environment;
 use super::FilePermissions;
 use super::UrlDownloader;
 use crate::plugins::CompilationResult;
+use crate::utils::get_bytes_hash;
 use crate::utils::LogLevel;
 
 #[derive(Default)]
@@ -97,7 +100,9 @@ impl Write for TestPipeWriter {
 pub struct TestEnvironment {
   log_level: Arc<Mutex<LogLevel>>,
   cwd: Arc<Mutex<String>>,
+  env_vars: Arc<Mutex<HashMap<String, OsString>>>,
   files: Arc<Mutex<HashMap<PathBuf, Vec<u8>>>>,
+  staged_files: Arc<Mutex<Vec<PathBuf>>>,
   file_permissions: Arc<Mutex<HashMap<PathBuf, FilePermissions>>>,
   stdout_messages: Arc<Mutex<Vec<String>>>,
   stderr_messages: Arc<Mutex<Vec<String>>>,
@@ -107,7 +112,6 @@ pub struct TestEnvironment {
   multi_selection_result: Arc<Mutex<Option<Vec<usize>>>>,
   confirm_results: Arc<Mutex<Vec<Result<Option<bool>>>>>,
   is_stdout_machine_readable: Arc<Mutex<bool>>,
-  wasm_compile_result: Arc<Mutex<Option<CompilationResult>>>,
   dir_info_error: Arc<Mutex<Option<std::io::Error>>>,
   std_in_pipe: Arc<Mutex<(Option<TestPipeWriter>, TestPipeReader)>>,
   std_out_pipe: Arc<Mutex<(Option<TestPipeWriter>, TestPipeReader)>>,
@@ -123,7 +127,9 @@ impl TestEnvironment {
     TestEnvironment {
       log_level: Arc::new(Mutex::new(LogLevel::Info)),
       cwd: Arc::new(Mutex::new(String::from("/"))),
+      env_vars: Default::default(),
       files: Default::default(),
+      staged_files: Default::default(),
       file_permissions: Default::default(),
       stdout_messages: Default::default(),
       stderr_messages: Default::default(),
@@ -133,7 +139,6 @@ impl TestEnvironment {
       multi_selection_result: Arc::new(Mutex::new(None)),
       confirm_results: Default::default(),
       is_stdout_machine_readable: Arc::new(Mutex::new(false)),
-      wasm_compile_result: Arc::new(Mutex::new(None)),
       dir_info_error: Arc::new(Mutex::new(None)),
       std_in_pipe: Arc::new(Mutex::new({
         let pipe = create_test_pipe();
@@ -189,6 +194,18 @@ impl TestEnvironment {
     self.deleted_directories.lock().contains(&path.as_ref().to_path_buf())
   }
 
+  pub fn set_env_var(&self, name: &str, value: Option<&str>) {
+    let mut env_vars = self.env_vars.lock();
+    match value {
+      Some(value) => {
+        env_vars.insert(name.to_string(), OsString::from(value));
+      }
+      None => {
+        env_vars.remove(name);
+      }
+    }
+  }
+
   pub fn set_selection_result(&self, index: usize) {
     *self.selection_result.lock() = index;
   }
@@ -214,10 +231,6 @@ impl TestEnvironment {
     *self.log_level.lock() = value;
   }
 
-  pub fn set_wasm_compile_result(&self, value: CompilationResult) {
-    *self.wasm_compile_result.lock() = Some(value);
-  }
-
   pub fn stdout_reader(&self) -> Box<dyn Read + Send> {
     Box::new(self.std_out_pipe.lock().1.clone())
   }
@@ -231,6 +244,9 @@ impl TestEnvironment {
     self.path_dirs.lock().clone()
   }
 
+  pub fn set_staged_file(&self, file: impl AsRef<Path>) {
+    self.staged_files.lock().push(file.as_ref().to_path_buf())
+  }
   pub fn set_dir_info_error(&self, err: std::io::Error) {
     *self.dir_info_error.lock() = Some(err);
   }
@@ -298,6 +314,14 @@ impl Environment for TestEnvironment {
     false
   }
 
+  fn env_var(&self, name: &str) -> Option<OsString> {
+    self.env_vars.lock().get(name).cloned()
+  }
+
+  fn get_staged_files(&self) -> Result<Vec<PathBuf>> {
+    Ok(self.staged_files.lock().clone())
+  }
+
   fn read_file(&self, file_path: impl AsRef<Path>) -> Result<String> {
     let file_bytes = self.read_file_bytes(file_path)?;
     Ok(String::from_utf8(file_bytes.to_vec()).unwrap())
@@ -473,6 +497,10 @@ impl Environment for TestEnvironment {
     self.canonicalize("/cache").unwrap()
   }
 
+  fn get_home_dir(&self) -> Option<CanonicalizedPathBuf> {
+    self.canonicalize("/home").ok()
+  }
+
   fn cpu_arch(&self) -> String {
     self.cpu_arch.lock().clone()
   }
@@ -531,9 +559,26 @@ impl Environment for TestEnvironment {
     *self.log_level.lock()
   }
 
-  fn compile_wasm(&self, _: &[u8]) -> Result<CompilationResult> {
-    let wasm_compile_result = self.wasm_compile_result.lock();
-    Ok(wasm_compile_result.clone().expect("Expected compilation result to be set."))
+  fn compile_wasm(&self, bytes: &[u8]) -> Result<CompilationResult> {
+    use std::collections::hash_map::Entry;
+
+    static COMPILE_RESULTS: Lazy<Mutex<HashMap<u64, CompilationResult>>> = Lazy::new(Default::default);
+
+    let hash = get_bytes_hash(bytes);
+    {
+      // hold the lock while compiling in order to prevent other
+      // threads from compiling at the same time
+      let mut results = COMPILE_RESULTS.lock();
+      let entry = results.entry(hash);
+      match entry {
+        Entry::Occupied(entry) => Ok(entry.get().clone()),
+        Entry::Vacant(entry) => {
+          let value = crate::plugins::compile_wasm(bytes).unwrap();
+          entry.insert(value.clone());
+          Ok(value)
+        }
+      }
+    }
   }
 
   fn wasm_cache_key(&self) -> String {
