@@ -1,3 +1,4 @@
+use indexmap::IndexMap;
 use serde::Serialize;
 use std::collections::HashMap;
 use std::path::Path;
@@ -7,6 +8,7 @@ use super::Environment;
 use super::TestEnvironment;
 use crate::test_helpers;
 use crate::test_helpers::TestProcessPluginFile;
+use crate::test_helpers::WASM_PLUGIN_0_1_0_BYTES;
 use crate::utils::get_sha256_checksum;
 
 pub struct TestConfigFileBuilder {
@@ -15,7 +17,7 @@ pub struct TestConfigFileBuilder {
   includes: Option<Vec<String>>,
   excludes: Option<Vec<String>>,
   plugins: Option<Vec<String>>,
-  sections: HashMap<String, String>,
+  sections: IndexMap<String, String>,
 }
 
 impl TestConfigFileBuilder {
@@ -26,7 +28,7 @@ impl TestConfigFileBuilder {
       includes: None,
       excludes: None,
       plugins: None,
-      sections: HashMap::new(),
+      sections: Default::default(),
     }
   }
 
@@ -74,6 +76,18 @@ impl TestConfigFileBuilder {
     self.add_plugin(&format!("https://plugins.dprint.dev/test-plugin.wasm@{}", checksum))
   }
 
+  /// This is a v3 old Wasm plugin.
+  pub fn add_remote_wasm_plugin_0_1_0(&mut self) -> &mut Self {
+    self.add_plugin("https://plugins.dprint.dev/test-plugin-0.1.0.wasm")
+  }
+
+  pub fn add_remote_wasm_plugin_0_1_0_with_checksum(&mut self) -> &mut Self {
+    self.add_plugin(&format!(
+      "https://plugins.dprint.dev/test-plugin-0.1.0.wasm@{}",
+      &crate::utils::get_sha256_checksum(&WASM_PLUGIN_0_1_0_BYTES)
+    ))
+  }
+
   pub fn add_config_section(&mut self, name: &str, text: &str) -> &mut Self {
     self.sections.insert(name.to_string(), text.to_string());
     self
@@ -171,7 +185,7 @@ impl TestInfoFileBuilder {
   pub fn to_string(&self) -> String {
     let mut parts = Vec::new();
     parts.push("\"schemaVersion\": 4".to_string());
-    parts.push(format!("\"pluginSystemSchemaVersion\": {}", self.plugin_schema_version.unwrap_or(3)));
+    parts.push(format!("\"pluginSystemSchemaVersion\": {}", self.plugin_schema_version.unwrap_or(4)));
     let plugins_text = serde_json::to_string_pretty(&self.plugins).unwrap();
     parts.push(format!("\"latest\": {}", plugins_text));
     format!("{{\n{}\n}}", parts.join(",\n"))
@@ -254,6 +268,13 @@ impl TestEnvironmentBuilder {
     self
   }
 
+  pub fn add_remote_wasm_0_1_0_plugin(&mut self) -> &mut Self {
+    self
+      .environment
+      .add_remote_file("https://plugins.dprint.dev/test-plugin-0.1.0.wasm", test_helpers::WASM_PLUGIN_0_1_0_BYTES);
+    self
+  }
+
   pub fn add_remote_wasm_plugin_at_url(&mut self, url: &str) -> &mut Self {
     self.environment.add_remote_file(url, test_helpers::WASM_PLUGIN_BYTES);
     self
@@ -299,6 +320,11 @@ impl TestEnvironmentBuilder {
     self
   }
 
+  pub fn add_staged_file(&mut self, file_path: impl AsRef<Path>) -> &mut Self {
+    self.environment.set_staged_file(file_path);
+    self
+  }
+
   pub fn add_remote_file(&mut self, path: &str, text: &str) -> &mut Self {
     self.environment.add_remote_file_bytes(path, text.to_string().into_bytes());
     self
