@@ -1,3 +1,4 @@
+use anyhow::Context;
 use anyhow::Result;
 use std::collections::HashMap;
 use std::collections::HashSet;
@@ -5,6 +6,7 @@ use std::path::PathBuf;
 use std::str::Split;
 use thiserror::Error;
 
+use crate::arg_parser::ConfigDiscovery;
 use crate::arg_parser::FilePatternArgs;
 use crate::configuration::ResolvedConfig;
 use crate::environment::CanonicalizedPathBuf;
@@ -86,21 +88,37 @@ pub fn get_file_paths_by_plugins(plugin_name_maps: &PluginNameResolutionMaps, fi
 pub async fn get_and_resolve_file_paths<'a>(
   config: &ResolvedConfig,
   args: &FilePatternArgs,
+  config_discovery: ConfigDiscovery,
   plugins: impl Iterator<Item = &'a PluginWithConfig>,
   environment: &impl Environment,
 ) -> Result<GlobOutput> {
   let cwd = environment.cwd();
   let mut file_patterns = get_all_file_patterns(config, args, &cwd);
+
+  if args.only_staged {
+    let staged_files = environment.get_staged_files().context("Failed running git staged.")?;
+    file_patterns.arg_includes = Some(GlobPattern::new_vec(
+      staged_files.into_iter().map(|path| path.to_string_lossy().into_owned()).collect(),
+      cwd.clone(),
+    ));
+  }
+
   if file_patterns.config_includes.is_none() {
     // If no includes patterns were specified, derive one from the list of plugins
     // as this is a massive performance improvement, because it collects less file
     // paths to examine and match to plugins later.
     file_patterns.config_includes = Some(GlobPattern::new_vec(get_plugin_patterns(plugins), cwd.clone()));
   }
-  get_and_resolve_file_patterns(config, file_patterns, environment).await
+
+  get_and_resolve_file_patterns(config, file_patterns, config_discovery, environment).await
 }
 
-async fn get_and_resolve_file_patterns<'a>(config: &ResolvedConfig, file_patterns: GlobPatterns, environment: &impl Environment) -> Result<GlobOutput> {
+async fn get_and_resolve_file_patterns(
+  config: &ResolvedConfig,
+  file_patterns: GlobPatterns,
+  config_discovery: ConfigDiscovery,
+  environment: &impl Environment,
+) -> Result<GlobOutput> {
   let cwd = environment.cwd();
   let is_cwd_in_base = cwd.starts_with(&config.base_path);
   let is_in_sub_dir = cwd != config.base_path && is_cwd_in_base;
@@ -116,6 +134,7 @@ async fn get_and_resolve_file_patterns<'a>(config: &ResolvedConfig, file_pattern
         start_dir: start_dir.into_path_buf(),
         file_patterns,
         pattern_base,
+        config_discovery,
       },
     )
   })
