@@ -5,15 +5,14 @@ use dprint_core::async_runtime::FutureExt;
 use dprint_core::async_runtime::LocalBoxFuture;
 use dprint_core::plugins::process::HostFormatCallback;
 use dprint_core::plugins::CancellationToken;
+use dprint_core::plugins::CheckConfigUpdatesMessage;
 use dprint_core::plugins::ConfigChange;
 use dprint_core::plugins::CriticalFormatError;
 use dprint_core::plugins::FileMatchingInfo;
-use dprint_core::plugins::FormatConfigId;
+use dprint_core::plugins::FormatRange;
 use dprint_core::plugins::FormatResult;
 use dprint_core::plugins::HostFormatRequest;
-use dprint_core::plugins::SyncPluginInfo;
 use std::cell::RefCell;
-use std::path::Path;
 use std::path::PathBuf;
 use std::rc::Rc;
 use std::sync::Arc;
@@ -21,18 +20,16 @@ use std::time::Instant;
 
 use dprint_core::configuration::ConfigKeyMap;
 use dprint_core::configuration::ConfigurationDiagnostic;
-use dprint_core::configuration::GlobalConfiguration;
 use dprint_core::plugins::PluginInfo;
 
 use super::create_pools_import_object;
 use super::load_instance;
 use super::load_instance::WasmInstance;
 use super::load_instance::WasmModule;
-use super::WasmFormatResult;
-use super::WasmFunctions;
 use super::WasmHostFormatSender;
 use super::WasmModuleCreator;
 use crate::environment::Environment;
+use crate::plugins::implementations::wasm::create_wasm_plugin_instance;
 use crate::plugins::FormatConfig;
 use crate::plugins::InitializedPlugin;
 use crate::plugins::InitializedPluginFormatRequest;
@@ -66,15 +63,15 @@ impl<TEnvironment: Environment> Plugin for WasmPlugin<TEnvironment> {
   }
 
   async fn initialize(&self) -> Result<Rc<dyn InitializedPlugin>> {
+    let environment = self.environment.clone();
+    let plugin_name = self.info().name.clone();
     let plugin: Rc<dyn InitializedPlugin> = Rc::new(InitializedWasmPlugin::new(
-      self.info().name.to_string(),
+      plugin_name.clone(),
       self.module.clone(),
       Arc::new({
         move |store, module, host_format_sender| {
-          let (import_object, env) = create_pools_import_object(store, host_format_sender);
-          let instance = load_instance(store, module, &import_object)?;
-          env.as_mut(store).initialize(&instance.inner)?;
-          Ok(instance)
+          let (import_object, env) = create_pools_import_object(environment.clone(), &plugin_name, module.version(), store, host_format_sender);
+          load_instance(store, module, env, &import_object)
         }
       }),
       self.environment.clone(),
@@ -87,8 +84,10 @@ impl<TEnvironment: Environment> Plugin for WasmPlugin<TEnvironment> {
 struct WasmPluginFormatMessage {
   file_path: PathBuf,
   file_bytes: Vec<u8>,
+  range: FormatRange,
   config: Arc<FormatConfig>,
   override_config: ConfigKeyMap,
+  token: Arc<dyn CancellationToken>,
 }
 
 type WasmResponseSender<T> = tokio::sync::oneshot::Sender<T>;
@@ -96,6 +95,7 @@ type WasmResponseSender<T> = tokio::sync::oneshot::Sender<T>;
 enum WasmPluginMessage {
   LicenseText(WasmResponseSender<Result<String>>),
   ResolvedConfig(Arc<FormatConfig>, WasmResponseSender<Result<String>>),
+  CheckConfigUpdates(Arc<CheckConfigUpdatesMessage>, WasmResponseSender<Result<Vec<ConfigChange>>>),
   FileMatchingInfo(Arc<FormatConfig>, WasmResponseSender<Result<FileMatchingInfo>>),
   ConfigDiagnostics(Arc<FormatConfig>, WasmResponseSender<Result<Vec<ConfigurationDiagnostic>>>),
   FormatRequest(Arc<WasmPluginFormatMessage>, WasmResponseSender<FormatResult>),
@@ -106,7 +106,6 @@ type WasmPluginSender = std::sync::mpsc::Sender<WasmPluginMessage>;
 #[derive(Clone)]
 struct InstanceState {
   host_format_callback: HostFormatCallback,
-  token: Arc<dyn CancellationToken>,
 }
 
 struct WasmPluginSenderWithState {
@@ -114,177 +113,6 @@ struct WasmPluginSenderWithState {
   instance_state_cell: Rc<RefCell<Option<InstanceState>>>,
 }
 
-pub(super) struct InitializedWasmPluginInstance {
-  wasm_functions: WasmFunctions,
-  buffer_size: usize,
-  current_config_id: FormatConfigId,
-}
-
-impl InitializedWasmPluginInstance {
-  pub fn new(mut wasm_functions: WasmFunctions) -> Result<Self> {
-    let buffer_size = wasm_functions.get_wasm_memory_buffer_size()?;
-    Ok(Self {
-      wasm_functions,
-      buffer_size,
-      current_config_id: FormatConfigId::uninitialized(),
-    })
-  }
-
-  pub fn set_global_config(&mut self, global_config: &GlobalConfiguration) -> Result<()> {
-    let json = serde_json::to_string(global_config)?;
-    self.send_string(&json)?;
-    self.wasm_functions.set_global_config()?;
-    Ok(())
-  }
-
-  pub fn set_plugin_config(&mut self, plugin_config: &ConfigKeyMap) -> Result<()> {
-    let json = serde_json::to_string(plugin_config)?;
-    self.send_string(&json)?;
-    self.wasm_functions.set_plugin_config()?;
-    Ok(())
-  }
-
-  pub fn plugin_info(&mut self) -> Result<PluginInfo> {
-    self.sync_plugin_info().map(|i| i.info)
-  }
-
-  fn sync_plugin_info(&mut self) -> Result<SyncPluginInfo> {
-    let len = self.wasm_functions.get_plugin_info()?;
-    let json_text = self.receive_string(len)?;
-    Ok(serde_json::from_str(&json_text)?)
-  }
-
-  fn license_text(&mut self) -> Result<String> {
-    let len = self.wasm_functions.get_license_text()?;
-    self.receive_string(len)
-  }
-
-  fn resolved_config(&mut self, config: &FormatConfig) -> Result<String> {
-    self.ensure_config(config)?;
-    let len = self.wasm_functions.get_resolved_config()?;
-    self.receive_string(len)
-  }
-
-  fn config_diagnostics(&mut self, config: &FormatConfig) -> Result<Vec<ConfigurationDiagnostic>> {
-    self.ensure_config(config)?;
-    let len = self.wasm_functions.get_config_diagnostics()?;
-    let json_text = self.receive_string(len)?;
-    Ok(serde_json::from_str(&json_text)?)
-  }
-
-  fn file_matching_info(&mut self, _config: &FormatConfig) -> Result<FileMatchingInfo> {
-    self.sync_plugin_info().map(|i| i.file_matching)
-  }
-
-  fn format_text(&mut self, file_path: &Path, file_bytes: &[u8], config: &FormatConfig, override_config: &ConfigKeyMap) -> FormatResult {
-    self.ensure_config(config)?;
-    match self.inner_format_text(file_path, file_bytes, override_config) {
-      Ok(inner) => inner,
-      Err(err) => Err(CriticalFormatError(err).into()),
-    }
-  }
-
-  fn inner_format_text(&mut self, file_path: &Path, file_bytes: &[u8], override_config: &ConfigKeyMap) -> Result<FormatResult> {
-    // send override config if necessary
-    if !override_config.is_empty() {
-      self.send_string(&match serde_json::to_string(override_config) {
-        Ok(text) => text,
-        Err(err) => return Ok(Err(err.into())),
-      })?;
-      self.wasm_functions.set_override_config()?;
-    }
-
-    // send file path
-    self.send_string(&file_path.to_string_lossy())?;
-    self.wasm_functions.set_file_path()?;
-
-    // send file text and format
-    self.send_bytes(file_bytes)?;
-    let response_code = self.wasm_functions.format()?;
-
-    // handle the response
-    match response_code {
-      WasmFormatResult::NoChange => Ok(Ok(None)),
-      WasmFormatResult::Change => {
-        let len = self.wasm_functions.get_formatted_text()?;
-        let text_bytes = self.receive_bytes(len)?;
-        Ok(Ok(Some(text_bytes)))
-      }
-      WasmFormatResult::Error => {
-        let len = self.wasm_functions.get_error_text()?;
-        let text = self.receive_string(len)?;
-        Ok(Err(anyhow!("{}", text)))
-      }
-    }
-  }
-
-  fn ensure_config(&mut self, config: &FormatConfig) -> Result<()> {
-    if self.current_config_id != config.id {
-      // set this to uninitialized in case it errors below
-      self.current_config_id = FormatConfigId::uninitialized();
-      // update the plugin
-      self.set_global_config(&config.global)?;
-      self.set_plugin_config(&config.raw)?;
-      // now mark this as successfully set
-      self.current_config_id = config.id;
-    }
-    Ok(())
-  }
-
-  /* LOW LEVEL SENDING AND RECEIVING */
-
-  // These methods should panic when failing because that may indicate
-  // a major problem where the CLI is out of sync with the plugin.
-
-  fn send_string(&mut self, text: &str) -> Result<()> {
-    self.send_bytes(text.as_bytes())
-  }
-
-  fn send_bytes(&mut self, bytes: &[u8]) -> Result<()> {
-    let mut index = 0;
-    let len = bytes.len();
-    self.wasm_functions.clear_shared_bytes(len)?;
-    while index < len {
-      let write_count = std::cmp::min(len - index, self.buffer_size);
-      self.write_bytes_to_memory_buffer(&bytes[index..(index + write_count)])?;
-      self.wasm_functions.add_to_shared_bytes_from_buffer(write_count)?;
-      index += write_count;
-    }
-    Ok(())
-  }
-
-  fn write_bytes_to_memory_buffer(&mut self, bytes: &[u8]) -> Result<()> {
-    let wasm_buffer_pointer = self.wasm_functions.get_wasm_memory_buffer_ptr()?;
-    let memory_view = self.wasm_functions.get_memory_view();
-    memory_view.write(wasm_buffer_pointer.offset() as u64, bytes)?;
-    Ok(())
-  }
-
-  fn receive_string(&mut self, len: usize) -> Result<String> {
-    let bytes = self.receive_bytes(len)?;
-    Ok(String::from_utf8(bytes)?)
-  }
-
-  fn receive_bytes(&mut self, len: usize) -> Result<Vec<u8>> {
-    let mut index = 0;
-    let mut bytes: Vec<u8> = vec![0; len];
-    while index < len {
-      let read_count = std::cmp::min(len - index, self.buffer_size);
-      self.wasm_functions.set_buffer_with_shared_bytes(index, read_count)?;
-      self.read_bytes_from_memory_buffer(&mut bytes[index..(index + read_count)])?;
-      index += read_count;
-    }
-    Ok(bytes)
-  }
-
-  fn read_bytes_from_memory_buffer(&mut self, bytes: &mut [u8]) -> Result<()> {
-    let wasm_buffer_pointer = self.wasm_functions.get_wasm_memory_buffer_ptr()?;
-    let memory_view = self.wasm_functions.get_memory_view();
-    memory_view.read(wasm_buffer_pointer.offset() as u64, bytes)?;
-    Ok(())
-  }
-}
-
 type LoadInstanceFn = dyn Fn(&mut wasmer::Store, &WasmModule, WasmHostFormatSender) -> Result<WasmInstance> + Send + Sync;
 
 pub struct InitializedWasmPlugin<TEnvironment: Environment> {
@@ -407,12 +235,10 @@ impl<TEnvironment: Environment> InitializedWasmPlugin<TEnvironment> {
     dprint_core::async_runtime::spawn({
       let instance_state_cell = instance_state_cell.clone();
       async move {
-        while let Some((mut request, sender)) = host_format_rx.recv().await {
+        while let Some((request, sender)) = host_format_rx.recv().await {
           let instance_state = instance_state_cell.borrow().clone();
           match instance_state {
             Some(instance_state) => {
-              // forward the provided token on to the host format
-              request.token = instance_state.token.clone();
               let message = (instance_state.host_format_callback)(request).await;
               if sender.send(message).is_err() {
                 return; // disconnected
@@ -438,8 +264,7 @@ impl<TEnvironment: Environment> InitializedWasmPlugin<TEnvironment> {
       move || {
         let initialize = || {
           let instance = (load_instance)(&mut store, &module, host_format_tx)?;
-          let wasm_functions = WasmFunctions::new(store, instance)?;
-          let instance = InitializedWasmPluginInstance::new(wasm_functions)?;
+          let instance = create_wasm_plugin_instance(store, instance)?;
           Ok(instance)
         };
         let mut instance = match initialize() {
@@ -462,6 +287,12 @@ impl<TEnvironment: Environment> InitializedWasmPlugin<TEnvironment> {
                 break; // disconnected
               }
             }
+            WasmPluginMessage::CheckConfigUpdates(message, response) => {
+              let result = instance.check_config_updates(&message);
+              if response.send(result).is_err() {
+                break; // disconnected
+              }
+            }
             WasmPluginMessage::ConfigDiagnostics(config, response) => {
               let result = instance.config_diagnostics(&config);
               if response.send(result).is_err() {
@@ -481,7 +312,14 @@ impl<TEnvironment: Environment> InitializedWasmPlugin<TEnvironment> {
               }
             }
             WasmPluginMessage::FormatRequest(request, response) => {
-              let result = instance.format_text(&request.file_path, &request.file_bytes, &request.config, &request.override_config);
+              let result = instance.format_text(
+                &request.file_path,
+                &request.file_bytes,
+                request.range.clone(),
+                &request.config,
+                &request.override_config,
+                request.token.clone(),
+              );
               if response.send(result).is_err() {
                 break; // disconnected
               }
@@ -564,28 +402,35 @@ impl<TEnvironment: Environment> InitializedPlugin for InitializedWasmPlugin<TEnv
       .await
   }
 
-  async fn check_config_updates(&self, _plugin_config: ConfigKeyMap) -> Result<Vec<ConfigChange>> {
-    Ok(Vec::new()) // not supported atm
+  async fn check_config_updates(&self, message: CheckConfigUpdatesMessage) -> Result<Vec<ConfigChange>> {
+    let message = Arc::new(message);
+    self
+      .with_instance(None, move |plugin_sender| {
+        let message = message.clone();
+        async move {
+          let (tx, rx) = tokio::sync::oneshot::channel();
+          plugin_sender.send(WasmPluginMessage::CheckConfigUpdates(message, tx))?;
+          rx.await?
+        }
+        .boxed_local()
+      })
+      .await
   }
 
   async fn format_text(&self, request: InitializedPluginFormatRequest) -> FormatResult {
-    // Wasm plugins do not currently support range formatting
-    // so always return back None for now.
-    if request.range.is_some() {
-      return Ok(None);
-    }
     if request.token.is_cancelled() {
       return Ok(None);
     }
     let message = Arc::new(WasmPluginFormatMessage {
       file_path: request.file_path,
       file_bytes: request.file_text,
+      range: request.range,
       config: request.config,
       override_config: request.override_config,
+      token: request.token,
     });
     let instance_state = InstanceState {
       host_format_callback: request.on_host_format,
-      token: request.token,
     };
     self
       .with_instance(Some(instance_state), move |plugin_sender| {
