@@ -13,6 +13,7 @@ use dprint_core::async_runtime::LocalBoxFuture;
 use dprint_core::configuration::ConfigKeyMap;
 use dprint_core::plugins::process::HostFormatCallback;
 use dprint_core::plugins::CancellationToken;
+use dprint_core::plugins::CheckConfigUpdatesMessage;
 use dprint_core::plugins::ConfigChange;
 use dprint_core::plugins::CriticalFormatError;
 use dprint_core::plugins::FileMatchingInfo;
@@ -24,6 +25,7 @@ use indexmap::IndexMap;
 use thiserror::Error;
 
 use crate::arg_parser::CliArgs;
+use crate::arg_parser::ConfigDiscovery;
 use crate::arg_parser::FilePatternArgs;
 use crate::configuration::get_global_config;
 use crate::configuration::get_plugin_config_map;
@@ -84,7 +86,7 @@ impl PluginWithConfig {
     hasher.write(self.info().version.as_bytes());
 
     // serialize the config keys in order to prevent the hash from changing
-    let sorted_config = self.format_config.raw.iter().collect::<BTreeMap<_, _>>();
+    let sorted_config = self.format_config.plugin.iter().collect::<BTreeMap<_, _>>();
     for (key, value) in sorted_config {
       hasher.write(key.as_bytes());
       value.hash(hasher);
@@ -179,8 +181,8 @@ impl InitializedPluginWithConfig {
     output_plugin_config_diagnostics(&self.info().name, &*self.instance, self.plugin.format_config.clone(), environment).await
   }
 
-  pub async fn check_config_updates(&self, plugin_config: ConfigKeyMap) -> Result<Vec<ConfigChange>> {
-    self.instance.check_config_updates(plugin_config).await
+  pub async fn check_config_updates(&self, message: CheckConfigUpdatesMessage) -> Result<Vec<ConfigChange>> {
+    self.instance.check_config_updates(message).await
   }
 
   pub async fn format_text(&self, request: InitializedPluginWithConfigFormatRequest) -> FormatResult {
@@ -205,7 +207,7 @@ pub struct PluginsScope<TEnvironment: Environment> {
   pub plugins: IndexMap<String, Rc<PluginWithConfig>>,
   pub plugin_name_maps: PluginNameResolutionMaps,
   global_config_diagnostics: Vec<GlobalConfigDiagnostic>,
-  cached_editor_file_matcher: RefCell<Option<FileMatcher>>,
+  cached_editor_file_matcher: RefCell<Option<FileMatcher<TEnvironment>>>,
 }
 
 impl<TEnvironment: Environment> PluginsScope<TEnvironment> {
@@ -326,7 +328,7 @@ impl<TEnvironment: Environment> PluginsScope<TEnvironment> {
       let Some(config) = &self.config else {
         return false;
       };
-      let matcher = match FileMatcher::new(config, &FilePatternArgs::default(), &config.base_path) {
+      let matcher = match FileMatcher::new(self.environment.clone(), config, &FilePatternArgs::default(), &config.base_path) {
         Ok(matcher) => matcher,
         Err(err) => {
           log_warn!(self.environment, "Error creating file matcher: {}", err);
@@ -335,7 +337,7 @@ impl<TEnvironment: Environment> PluginsScope<TEnvironment> {
       };
       file_matcher_borrow.replace(matcher);
     }
-    match file_matcher_borrow.as_ref() {
+    match file_matcher_borrow.as_mut() {
       Some(file_matcher) => file_matcher.matches_and_dir_not_ignored(file_path),
       None => false, // should never happen
     }
@@ -423,6 +425,10 @@ impl<TEnvironment: Environment> PluginsScopeAndPathsCollection<TEnvironment> {
     Ok(())
   }
 
+  pub fn len(&self) -> usize {
+    self.inner.len()
+  }
+
   pub fn iter(&self) -> impl Iterator<Item = &PluginsScopeAndPaths<TEnvironment>> {
     self.inner.iter()
   }
@@ -464,14 +470,26 @@ impl<'a, TEnvironment: Environment> PluginsAndPathsResolver<'a, TEnvironment> {
   pub async fn resolve_for_config(&self) -> Result<PluginsScopeAndPathsCollection<TEnvironment>> {
     let config = Rc::new(resolve_config_from_args(self.args, self.environment).await?);
     let scope = resolve_plugins_scope(config.clone(), self.environment, self.plugin_resolver).await?;
-    let glob_output = get_and_resolve_file_paths(&config, self.patterns, scope.plugins.values().map(|p| p.as_ref()), self.environment).await?;
+    let config_discovery = self.args.config_discovery(self.environment);
+    let glob_output = get_and_resolve_file_paths(
+      &config,
+      self.patterns,
+      config_discovery,
+      scope.plugins.values().map(|p| p.as_ref()),
+      self.environment,
+    )
+    .await?;
     let file_paths_by_plugins = get_file_paths_by_plugins(&scope.plugin_name_maps, glob_output.file_paths)?;
 
     let mut result = vec![PluginsScopeAndPaths { scope, file_paths_by_plugins }];
     let root_config_path = config.resolved_path.source.maybe_local_path();
     // todo: parallelize?
     for config_file_path in glob_output.config_files {
-      result.extend(self.resolve_for_sub_config(config_file_path, &config, root_config_path).await?);
+      result.extend(
+        self
+          .resolve_for_sub_config(config_file_path, &config, config_discovery, root_config_path)
+          .await?,
+      );
     }
 
     Ok(PluginsScopeAndPathsCollection {
@@ -484,6 +502,7 @@ impl<'a, TEnvironment: Environment> PluginsAndPathsResolver<'a, TEnvironment> {
     &'a self,
     config_file_path: PathBuf,
     parent_config: &'a ResolvedConfig,
+    config_discovery: ConfigDiscovery,
     root_config_path: Option<&'a CanonicalizedPathBuf>,
   ) -> LocalBoxFuture<'a, Result<Vec<PluginsScopeAndPaths<TEnvironment>>>> {
     async move {
@@ -499,17 +518,28 @@ impl<'a, TEnvironment: Environment> PluginsAndPathsResolver<'a, TEnvironment> {
       };
       let mut config = resolve_config_from_path(&config_path, self.environment).await?;
       if !self.args.plugins.is_empty() {
-        config.plugins = parent_config.plugins.clone();
+        config.plugins.clone_from(&parent_config.plugins);
       }
       let config = Rc::new(config);
       let scope = resolve_plugins_scope(config.clone(), self.environment, self.plugin_resolver).await?;
-      let glob_output = get_and_resolve_file_paths(&config, self.patterns, scope.plugins.values().map(|p| p.as_ref()), self.environment).await?;
+      let glob_output = get_and_resolve_file_paths(
+        &config,
+        self.patterns,
+        config_discovery,
+        scope.plugins.values().map(|p| p.as_ref()),
+        self.environment,
+      )
+      .await?;
       let file_paths_by_plugins = get_file_paths_by_plugins(&scope.plugin_name_maps, glob_output.file_paths)?;
 
       let mut result = vec![PluginsScopeAndPaths { scope, file_paths_by_plugins }];
       // todo: parallelize?
       for config_file_path in glob_output.config_files {
-        result.extend(self.resolve_for_sub_config(config_file_path, &config, root_config_path).await?);
+        result.extend(
+          self
+            .resolve_for_sub_config(config_file_path, &config, config_discovery, root_config_path)
+            .await?,
+        );
       }
 
       Ok(result)
@@ -573,7 +603,7 @@ pub async fn resolve_plugins_scope<TEnvironment: Environment>(
       let format_config = Arc::new(FormatConfig {
         id: next_config_id,
         global: global_config,
-        raw: plugin_config.properties,
+        plugin: plugin_config.properties,
       });
       let file_matching_info = instance.file_matching_info(format_config.clone()).await?;
       Ok::<_, anyhow::Error>(Rc::new(PluginWithConfig::new(
