@@ -30,14 +30,14 @@ pub fn show_multi_select(logger: &Logger, context_name: &str, prompt: &str, item
 
     if let Event::Key(key_event) = read_terminal_key_press()? {
       match &key_event.code {
-        KeyCode::Up => {
+        KeyCode::Up | KeyCode::Char('k') => {
           if data.active_index == 0 {
             data.active_index = data.items.len() - 1;
           } else {
             data.active_index -= 1;
           }
         }
-        KeyCode::Down => {
+        KeyCode::Down | KeyCode::Char('j') => {
           data.active_index = (data.active_index + 1) % data.items.len();
         }
         KeyCode::Char(' ') => {
@@ -48,7 +48,7 @@ pub fn show_multi_select(logger: &Logger, context_name: &str, prompt: &str, item
         KeyCode::Enter => {
           break;
         }
-        KeyCode::Esc => {
+        KeyCode::Esc | KeyCode::Char('q') => {
           logger.remove_refresh_item(LoggerRefreshItemKind::Selection);
           bail!("Selection cancelled.");
         }
