@@ -22,17 +22,19 @@ pub enum ProgressBarStyle {
   Action,
 }
 
-#[derive(Clone)]
 pub struct ProgressBar {
   id: usize,
-  start_time: SystemTime,
-  progress_bars: ProgressBars,
-  message: String,
-  size: usize,
-  style: ProgressBarStyle,
+  logger: Arc<Logger>,
+  state: Arc<Mutex<InternalState>>,
   pos: Arc<RwLock<usize>>,
 }
 
+impl Drop for ProgressBar {
+  fn drop(&mut self) {
+    self.finish();
+  }
+}
+
 impl ProgressBar {
   pub fn set_position(&self, new_pos: usize) {
     let mut pos = self.pos.write();
@@ -40,21 +42,38 @@ impl ProgressBar {
   }
 
   pub fn finish(&self) {
-    self.progress_bars.finish_progress(self.id);
+    let mut internal_state = self.state.lock();
+
+    if let Some(index) = internal_state.progress_bars.iter().position(|p| p.id == self.id) {
+      internal_state.progress_bars.remove(index);
+
+      if internal_state.progress_bars.is_empty() {
+        self.logger.remove_refresh_item(LoggerRefreshItemKind::ProgressBars);
+        internal_state.drawer_id += 1;
+      }
+    }
   }
 }
 
-#[derive(Clone)]
 pub struct ProgressBars {
-  logger: Logger,
+  logger: Arc<Logger>,
   state: Arc<Mutex<InternalState>>,
 }
 
+struct ProgressBarState {
+  id: usize,
+  start_time: SystemTime,
+  message: String,
+  size: usize,
+  style: ProgressBarStyle,
+  pos: Arc<RwLock<usize>>,
+}
+
 struct InternalState {
   // this ensures only one draw thread is running
   drawer_id: usize,
   progress_bar_counter: usize,
-  progress_bars: Vec<ProgressBar>,
+  progress_bars: Vec<ProgressBarState>,
 }
 
 impl ProgressBars {
@@ -64,7 +83,7 @@ impl ProgressBars {
   }
 
   /// Creates a new ProgressBars or returns None when not supported.
-  pub fn new(logger: &Logger) -> Option<Self> {
+  pub fn new(logger: &Arc<Logger>) -> Option<Self> {
     if ProgressBars::are_supported() {
       Some(ProgressBars {
         logger: logger.clone(),
@@ -82,16 +101,22 @@ impl ProgressBars {
   pub fn add_progress(&self, message: String, style: ProgressBarStyle, total_size: usize) -> ProgressBar {
     let mut internal_state = self.state.lock();
     let id = internal_state.progress_bar_counter;
-    let pb = ProgressBar {
+    let pos = Arc::new(RwLock::new(0));
+    let pb_state = ProgressBarState {
       id,
-      progress_bars: self.clone(),
       start_time: SystemTime::now(),
       message,
       size: total_size,
       style,
-      pos: Arc::new(RwLock::new(0)),
+      pos: pos.clone(),
     };
-    internal_state.progress_bars.push(pb.clone());
+    let pb = ProgressBar {
+      id,
+      logger: self.logger.clone(),
+      state: self.state.clone(),
+      pos,
+    };
+    internal_state.progress_bars.push(pb_state);
     internal_state.progress_bar_counter += 1;
 
     if internal_state.progress_bars.len() == 1 {
@@ -101,25 +126,12 @@ impl ProgressBars {
     pb
   }
 
-  fn finish_progress(&self, progress_bar_id: usize) {
-    let mut internal_state = self.state.lock();
-
-    if let Some(index) = internal_state.progress_bars.iter().position(|p| p.id == progress_bar_id) {
-      internal_state.progress_bars.remove(index);
-
-      if internal_state.progress_bars.is_empty() {
-        self.logger.remove_refresh_item(LoggerRefreshItemKind::ProgressBars);
-        internal_state.drawer_id += 1;
-      }
-    }
-  }
-
   fn start_draw_thread(&self, internal_state: &mut InternalState) {
     internal_state.drawer_id += 1;
     let drawer_id = internal_state.drawer_id;
     let internal_state = self.state.clone();
     let logger = self.logger.clone();
-    tokio::task::spawn_blocking(move || {
+    dprint_core::async_runtime::spawn_blocking(move || {
       loop {
         {
           let internal_state = internal_state.lock();
