@@ -1,14 +1,16 @@
 mod cached_downloader;
+mod certs;
 mod checksums;
 mod error_count_logger;
 mod extract_zip;
 mod file_path_utils;
-mod file_text;
 mod get_bytes_hash;
 mod get_difference;
+mod gitignore;
 mod glob;
 mod lax_single_process_fs_flag;
 mod logging;
+mod no_proxy;
 mod path_source;
 mod pretty_print_json_text;
 mod process;
@@ -27,12 +29,13 @@ pub use checksums::*;
 pub use error_count_logger::*;
 pub use extract_zip::*;
 pub use file_path_utils::*;
-pub use file_text::*;
 pub use get_bytes_hash::*;
 pub use get_difference::*;
+pub use gitignore::*;
 pub use glob::*;
 pub use lax_single_process_fs_flag::*;
 pub use logging::*;
+pub use no_proxy::*;
 pub use path_source::*;
 pub use pretty_print_json_text::*;
 pub use process::*;
