@@ -6,16 +6,16 @@ use std::io::{self};
 pub use tests::TestStdInReader;
 
 pub trait StdInReader: Clone + Send + Sync {
-  fn read(&self) -> Result<String>;
+  fn read(&self) -> Result<Vec<u8>>;
 }
 
 #[derive(Default, Clone, Copy)]
 pub struct RealStdInReader;
 
 impl StdInReader for RealStdInReader {
-  fn read(&self) -> Result<String> {
-    let mut text = String::new();
-    io::stdin().read_to_string(&mut text)?;
+  fn read(&self) -> Result<Vec<u8>> {
+    let mut text = Vec::new();
+    io::stdin().read_to_end(&mut text)?;
     Ok(text)
   }
 }
@@ -28,19 +28,19 @@ mod tests {
 
   #[derive(Default, Clone)]
   pub struct TestStdInReader {
-    text: Arc<Mutex<Option<String>>>,
+    text: Arc<Mutex<Option<Vec<u8>>>>,
   }
 
   impl<S: ToString> From<S> for TestStdInReader {
     fn from(value: S) -> Self {
       Self {
-        text: Arc::new(Mutex::new(Some(value.to_string()))),
+        text: Arc::new(Mutex::new(Some(value.to_string().into_bytes()))),
       }
     }
   }
 
   impl StdInReader for TestStdInReader {
-    fn read(&self) -> Result<String> {
+    fn read(&self) -> Result<Vec<u8>> {
       let text = self.text.lock();
       Ok(text.as_ref().expect("Expected to have stdin text set.").clone())
     }
