@@ -36,7 +36,7 @@ module.exports = {
       // in order to make things faster the next time we run and to allow the
       // dprint vscode extension to easily pick this up, copy the executable
       // into the dprint package folder
-      atomicCopyFileSync(sourceExecutablePath, targetExecutablePath);
+      hardLinkOrCopy(sourceExecutablePath, targetExecutablePath);
       if (os.platform() !== "win32") {
         // chomd +x
         chmodX(targetExecutablePath);
@@ -81,8 +81,8 @@ function getTarget() {
 
 function getArch() {
   const arch = os.arch();
-  if (arch !== "arm64" && arch !== "x64") {
-    throw new Error("Unsupported architecture " + os.arch() + ". Only x64 and aarch64 binaries are available.");
+  if (arch !== "arm64" && arch !== "x64" && arch !== "riscv64") {
+    throw new Error("Unsupported architecture " + os.arch() + ". Only x64, aarch64, and riscv64 binaries are available.");
   }
   return arch;
 }
@@ -144,7 +144,19 @@ function getLinuxFamily() {
  * @param sourcePath {string}
  * @param destinationPath {string}
  */
-function atomicCopyFileSync(sourcePath, destinationPath) {
+function hardLinkOrCopy(sourcePath, destinationPath) {
+  try {
+    fs.linkSync(sourcePath, destinationPath);
+  } catch {
+    atomicCopyFile(sourcePath, destinationPath);
+  }
+}
+
+/**
+ * @param sourcePath {string}
+ * @param destinationPath {string}
+ */
+function atomicCopyFile(sourcePath, destinationPath) {
   const crypto = require("crypto");
   const rand = crypto.randomBytes(4).toString("hex");
   const tempFilePath = destinationPath + "." + rand;
