@@ -7,6 +7,10 @@ const dockerfileUrl = "https://plugins.dprint.dev/dockerfile-x.x.x.wasm";
 const biomeUrl = "https://plugins.dprint.dev/biome-x.x.x.wasm";
 const ruffUrl = "https://plugins.dprint.dev/ruff-x.x.x.wasm";
 const jupyterUrl = "https://plugins.dprint.dev/jupyter-x.x.x.wasm";
+const malvaUrl = "https://plugins.dprint.dev/g-plane/malva-vx.x.x.wasm";
+const markupFmtUrl = "https://plugins.dprint.dev/g-plane/markup_fmt-vx.x.x.wasm";
+const yamlUrl = "https://plugins.dprint.dev/g-plane/pretty_yaml-vx.x.x.wasm";
+const graphqlUrl = "https://plugins.dprint.dev/g-plane/pretty_graphql-vx.x.x.wasm";
 const pluginInfoUrl = "https://plugins.dprint.dev/info.json";
 const schemaVersion = 4;
 
@@ -41,6 +45,18 @@ export function replacePluginUrls() {
           case getWithQuotes(jupyterUrl):
             element.textContent = getWithQuotes(urls["jupyter"]);
             break;
+          case getWithQuotes(malvaUrl):
+            element.textContent = getWithQuotes(urls["malva"]);
+            break;
+          case getWithQuotes(markupFmtUrl):
+            element.textContent = getWithQuotes(urls["markup_fmt"]);
+            break;
+          case getWithQuotes(yamlUrl):
+            element.textContent = getWithQuotes(urls["yaml"]);
+            break;
+          case getWithQuotes(graphqlUrl):
+            element.textContent = getWithQuotes(urls["graphql"]);
+            break;
         }
       }
     });
@@ -61,6 +77,10 @@ function getPluginUrlElements() {
       case getWithQuotes(biomeUrl):
       case getWithQuotes(ruffUrl):
       case getWithQuotes(jupyterUrl):
+      case getWithQuotes(malvaUrl):
+      case getWithQuotes(markupFmtUrl):
+      case getWithQuotes(yamlUrl):
+      case getWithQuotes(graphqlUrl):
         result.push(stringElement);
         break;
     }
@@ -91,6 +111,10 @@ function getPluginInfo() {
         biome: getUrlForPlugin(data, "dprint-plugin-biome"),
         ruff: getUrlForPlugin(data, "dprint-plugin-ruff"),
         jupyter: getUrlForPlugin(data, "dprint-plugin-jupyter"),
+        malva: getUrlForPlugin(data, "g-plane/malva"),
+        markup_fmt: getUrlForPlugin(data, "g-plane/markup_fmt"),
+        yaml: getUrlForPlugin(data, "g-plane/pretty_yaml"),
+        graphql: getUrlForPlugin(data, "g-plane/pretty_graphql"),
       };
     });
 
